@@ -35,6 +35,7 @@
 #include <QList>
 #include <QObject>
 
+#include "Contacts/group.h"
 #include "Contacts/contact.h"
 #include "Contacts/contactlist.h"
 
@@ -45,6 +46,8 @@ public:
     explicit RosterDBManager(QObject *parent = 0);
     ~RosterDBManager();
 
+    ContactList *getAllContacts();
+
 signals:
 
 public slots:
@@ -55,7 +58,11 @@ public slots:
     void updatePhoto(Contact *c);
     void updateStatus(Contact *c);
     void removeContact(QString jid);
-    ContactList *getAllContacts();
+    void removeGroup(QString gjid);
+    void addParticipant(Group *g, Contact *c);
+    void removeParticipant(Group *g, Contact *c);
+    void updateGroupSubject(Group *g);
+
 
 
 private:
