@@ -41,13 +41,19 @@ public:
     static QString decodeString(const char data[]);
     static QString getChatPassword();
     static QString getToken(QString phoneNumber);
-    static QString WATextToHtml(QString data, int iconSize = 32);
+    static QString WATextToHtml(QString data, int iconSize = 32, bool urlConversion = true);
     static QString htmlToWAText(QString html);
     static QString shortURL(QString url);
     static QString guessMimeType(QString extension);
     static QString getExtension(QString filename);
     static QString getPathFor(int media_wa_type, bool gallery);
     static QString formatBytes(qint64 bytes);
+    static QString removeEmoji(QString data);
+
+private:
+    static void initEmojiMapping();
+
+    static QHash<quint32,bool> emojiMapping;
 };
 
 #endif // UTILITIES_H
